'use strict';

/* Controllers */
(function(){

    var THRESHOLD = 250.0;
    var legendHtml = '<ul style="list-style: none;"><li style="float: left; margin-right: 10px;"><span style="border: 1px solid #ccc; float: left; width: 12px; height: 12px; margin: 2px; background-color:green;"></span>Consentito</li><li style="float: left; margin-right: 10px;"><span style="border: 1px solid #ccc; float: left; width: 12px; height: 12px; margin: 2px; background-color:red;"></span>Non consentito</li></ul>';

    var app = angular.module('firebulletin.controllers', []);

    app.controller('MainController', [
        '$scope', '$timeout', '$http', '$q', 'BulletinInfoProvider', 
        function($scope, $timeout, $http, $q, BulletinInfoProvider){

        this.error_message = "Servizio non disponibile";

        this.oInfo = null;
        this.sProtocolNumber = "";

        this.m_bLoading = true;
        this.m_bError = false;
        this.layers = [];

        var oDate = new Date();
        oDate.setHours(0);
        oDate.setMinutes(0);
        oDate.setSeconds(0);
        oDate.setMilliseconds(0);



        var sDate = experienceFormattedDate(oDate);
        this.sHumanDate = formattedDate(oDate);
        this.sDate = sDate;
        this.date = oDate;

        var $ctrl = this;
        $scope.$ctrl = this;

        BulletinInfoProvider
        .checkRun(this.sDate)
        .then(function(res){
            var p = $q.defer();
            if(res.data == "OK"){
                $ctrl.m_bLoading = true;
                p.resolve();
            }else{
                $ctrl.m_bLoading = false;
                $ctrl.m_bError = true;
                $ctrl.error_message = "Run non presente per la data odierna";
                p.reject();
            }
            return p.promise;
        }, function(err){
            console.log(err);
        }).then(function(){
            return $ctrl.downloadData();
        }).then(function(){
            $timeout(function(){
                $ctrl.initMaps();
            });            
        }, function(error){
            console.log(error);
            alert(error.data);
        });
    
        this.downloadData = function(){
            var p = $q.defer();
            var tried = 0;

            function download(){
                BulletinInfoProvider.getBulletinInfo($ctrl.sDate).then(function(res){
                    $ctrl.oInfo = res.data;
                    $ctrl.m_bLoading = false;
                    $ctrl.PDFURL = BulletinInfoProvider.getPDFURL($ctrl.sDate, $ctrl.sProtocolNumber);

                    var LAYERS_STR = 'layers=';
                    var ZDA_STR = ',firebulletin:puglia_zda';
                    var err = false;
                    angular.forEach($ctrl.oInfo.aoValues, function(val, i){
                        var img = val.image;
                        var first = img.indexOf(LAYERS_STR);
                        var last = img.indexOf(ZDA_STR);

                        if($ctrl.layers[i] == undefined){
                            if(first == -1 || last == -1 || last-first == LAYERS_STR.length){
                                err = true;
                            }else{
                                var layer = img.slice(first+LAYERS_STR.length, last);
                                $ctrl.layers[i] = layer;
                            }
                        }
                    });


                    if(err){
                        console.log('retry', tried);
                        tried++;
                        if(tried > 10){
                            p.reject();
                        }else{
                            download();                            
                        }
                    }else{
                        p.resolve();
                    }

                }, function(err){
                    if(tried < 10){
                        tried++;
                        download();
                    }
                });
                    
            }
            download();

            return p.promise;
        };



        this.getDay = function(val){
            var d = {
                '24H': ['Oggi', 0],
                '48H': ['Domani', 1],
                '72H': ['Dopodomani', 2]
            };

            var day = moment($ctrl.date);
            var n = day.add(d[val][1], 'd');
            
            return d[val][0] + " " + n.format('DD/MM/YYYY');
        }

        this.initMaps = function(){
            var highlightStyle = {
                stroke: true,
                color: '#2262CC', 
                weight: 3,
            };

            angular.forEach($ctrl.layers, function(layer, i){
                var map_id = 'map_' + i;
                var map = L.map(map_id, {
                        doubleClickZoom: false,
                        scrollWheelZoom: false,
                        touchZoom: false
                });

            var info = L.control();

            info.onAdd = function (map) {
                this._div = L.DomUtil.create('div', 'info'); // create a div with a class "info"
                this.update();
                return this._div;
            };


            // method that we will use to update the control based on feature properties passed
            info.update = function (props) {
                this._div.innerHTML = '<h4>Informazioni</h4>' +  (props ?
                    '<b>' + props.NOME_COM + '</b><br />' + (props.VALUE > THRESHOLD ?'Non Consentito':'Consentito')
                    : 'Passa il mouse su un comune <br>' + legendHtml);
            };

            info.addTo(map);

                L.tileLayer('http://{s}.tile.osm.org/{z}/{x}/{y}.png', {
                    attribution: '&copy; <a href="http://osm.org/copyright">OpenStreetMap</a> contributors'
                }).addTo(map);

                var layer = layer;
                BulletinInfoProvider.getGeoJson(layer).then(function(res){                        
                    var geojson = L.geoJSON(res.data, {
                        style: function (feature) {
                            return {
                                stroke: false,
                                fillColor: feature.properties.VALUE > THRESHOLD ? 'rgb(255,0,0)':'rgb(0,255,0)',
                                fillOpacity: 0.75
                            };
                        },
                        onEachFeature: function(feature, layer){
                            layer.on("mouseover", function (e) {
                                info.update(layer.feature.properties);
                                e.target.setStyle(highlightStyle);
                            });

                            layer.on("mouseout", function (e) {
                                info.update();
                                geojson.resetStyle(e.target);
                            });

                        }

                    });
                    geojson.addTo(map);
                    map.fitBounds(geojson.getBounds());

                }, function(err){
                    console.log('errore su mappa ' + i);
                });

            
            });
        };

        this.sendEmail = function(){
            BulletinInfoProvider.sendBulletinByEmail($ctrl.sDate, $ctrl.sProtocolNumber).success(function(data, status){
                if(data == 'OK') {
                    alert('Email Inviata con successo');
                }else{
                    alert('Errore di invio dell\'email');
                }
            }).error(function(data, status){
                alert(data);
            });
        }

        this.setLink = function(){
            $ctrl.PDFURL = BulletinInfoProvider.getPDFURL($ctrl.sDate, $ctrl.sProtocolNumber);
        }

}]);

function experienceFormattedDate(date) {
    var d = new Date(date || Date.now()),
        month = '' + (d.getMonth() + 1),
        day = '' + d.getDate(),
        year = d.getFullYear();

    if (month.length < 2) month = '0' + month;
    if (day.length < 2) day = '0' + day;

    return [year, month, day, '0000'].join('');
}

function formattedDate(date) {
    var d = new Date(date || Date.now()),
        month = '' + (d.getMonth() + 1),
        day = '' + d.getDate(),
        year = d.getFullYear();

    if (month.length < 2) month = '0' + month;
    if (day.length < 2) day = '0' + day;

    return [day, month, year].join('/');
}
})();